using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web.UI.HtmlControls;

namespace MessageBoardControl
{
	/// <summary>
	/// Summary description for AddCategory.
	/// </summary>
	[DefaultProperty("Text"), 
		ToolboxData("<{0}:AddCategory runat=server></{0}:AddCategory>")]
	public class AddCategory : System.Web.UI.WebControls.WebControl, INamingContainer
	{
		#region Properties
		private string text;
	
		[Bindable(true), 
			Category("Appearance"), 
			DefaultValue("")] 
		public string Text 
		{
			get
			{
				return text;
			}

			set
			{
				text = value;
			}
		}

		private string urlAfter;
		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public string URLAfter 
		{
			get
			{
				return urlAfter;
			}

			set
			{
				urlAfter = value;
			}
		}

		private MessageBoard.Group group;

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public MessageBoard.Group Group 
		{
			get
			{
				return group;
			}

			set
			{
				group = value;
			}
		}
		
		private MessageBoard.Language language;

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public MessageBoard.Language Language 
		{
			get
			{
				return language;
			}

			set
			{
				language = value;
			}
		}

		private MessageBoard.User currentUser;

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public MessageBoard.User CurrentUser 
		{
			get
			{
				return currentUser;
			}

			set
			{
				currentUser = value;
			}
		}
		#endregion

		#region Translate
		private MessageBoard.CustomControl customControl = MessageBoard.CustomControl.Get("AddCategory");

		private string Translate(string word)
		{
			return MessageBoard.Translation.Get(customControl, word, Language).Equal;
		}
		#endregion
		
		/// <summary> 
		/// Render this control to the output parameter specified.
		/// </summary>
		/// <param name="output"> The HTML writer to write out to </param>
		//protected override void Render(HtmlTextWriter output)
		//{
		//	output.Write(Text);
		//}

		protected override void CreateChildControls() 
		{
			// Check the required permissions
			if(MessageBoard.Role.Check(currentUser, group, MessageBoard.Role.Permission.CanAddCategories) != MessageBoard.Role.Allow.Yes)
			{
				return;
			}

			TextBox txtTitle = new TextBox();
			TextBox txtDescription = new TextBox();
			Button cmdAdd = new Button();
			
			txtTitle.Text ="";
			txtTitle.MaxLength = 100;
			txtTitle.Width = 500;
			txtTitle.ID="txtTitle";

			txtDescription.Text="";
			txtDescription.TextMode = TextBoxMode.MultiLine;
			txtDescription.Height = 100;
			txtDescription.Width = 500;
			txtDescription.ID = "txtDescription";

			cmdAdd.Text = Translate("Add");
			
			// Put the contols in a table.
			HtmlTable table = new HtmlTable();
			table.Attributes.Add("width", "100%");
			HtmlTableRow tr0 = new HtmlTableRow();
			table.Controls.Add(tr0);
			HtmlTableCell td0 = new HtmlTableCell();
			tr0.Controls.Add(td0);
			td0.Attributes.Add("class", "Forum_Title1");
			td0.InnerHtml = Translate("Add new category");

			HtmlTableRow tr01 = new HtmlTableRow();
			table.Controls.Add(tr01);
			HtmlTableCell td01 = new HtmlTableCell();
			tr01.Controls.Add(td01);
			td01.Attributes.Add("class", "Forum_Title4");

			// Table 2
			HtmlTable table2 = new HtmlTable();
			table2.Attributes.Add("width", "100%");

			// Title
			HtmlTableRow tr1 = new HtmlTableRow();
			table2.Controls.Add(tr1);
			HtmlTableCell td11 = new HtmlTableCell();
			HtmlTableCell td12 = new HtmlTableCell();
			td11.InnerHtml = Translate("Title")+":";
			td11.Attributes.Add("class", "Forum_Text");
			td12.Controls.Add(txtTitle);
			tr1.Controls.Add(td11);
			tr1.Controls.Add(td12);
			

			// Description
			HtmlTableRow tr2 = new HtmlTableRow();
			table2.Controls.Add(tr2);
			HtmlTableCell td21 = new HtmlTableCell();
			HtmlTableCell td22 = new HtmlTableCell();
			td21.InnerHtml = Translate("Description")+":";
			td21.Attributes.Add("class", "Forum_Text");
			td21.Attributes.Add("vAlign", "top");
			td22.Controls.Add(txtDescription);
			tr2.Controls.Add(td21);
			tr2.Controls.Add(td22);


			// Button
			HtmlTableRow tr3 = new HtmlTableRow();
			table2.Controls.Add(tr3);
			HtmlTableCell td31 = new HtmlTableCell();
			HtmlTableCell td32 = new HtmlTableCell();
			td31.InnerHtml = "";
			td32.Controls.Add(cmdAdd);
			tr3.Controls.Add(td31);
			tr3.Controls.Add(td32);

			td01.Controls.Add(table2);

			HtmlTable tableOut = new HtmlTable();
			HtmlTableRow trOut = new HtmlTableRow();
			HtmlTableCell tdOut = new HtmlTableCell();
			tableOut.BgColor = "#000000";
			tableOut.Attributes.Add("width", "100%");
			tableOut.CellPadding = 0;
			tableOut.CellSpacing = 0;
			table.CellSpacing = 1;
			tdOut.Controls.Add(table);
			trOut.Controls.Add(tdOut);
			tableOut.Controls.Add(trOut);
			Controls.Add(tableOut);


			cmdAdd.Click += new EventHandler(cmdAdd_Click);
                  
		}

		/// <summary>
		/// Add new Category
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public void cmdAdd_Click(Object sender, EventArgs e)
		{
			TextBox txtTitle = ((TextBox)FindControl("txtTitle"));
			TextBox txtDescription = ((TextBox)FindControl("txtDescription"));
			if(txtTitle.Text!="" || txtDescription.Text!="")
			{
				MessageBoard.Category category = new MessageBoard.Category();
				category.Title= txtTitle.Text;
				category.Description = txtDescription.Text;
				category.Group = group;
				category.Active = true;
				category.SortingOrder = 50;
				category.AddedBy = currentUser;
				category.AddedOn = DateTime.Now;
				category.Insert();

				txtTitle.Text="";
				txtDescription.Text="";
				this.EnsureChildControls();
				this.Page.Response.Redirect(urlAfter);
			}
		}

	}
}
