using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web.UI.HtmlControls;

namespace MessageBoardControl
{
	/// <summary>
	/// Summary description for AddGroup.
	/// </summary>
	[DefaultProperty("Text"), 
		ToolboxData("<{0}:AddGroup runat=server></{0}:AddGroup>")]
	public class EditGroup : Control, INamingContainer 
	{
		private string text;
	
		[Bindable(true), 
			Category("Appearance"), 
			DefaultValue("")] 
		public string Text 
		{
			get
			{
				return text;
			}

			set
			{
				text = value;
			}
		}

		private string urlAfter;
		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public string URLAfter 
		{
			get
			{
				return urlAfter;
			}

			set
			{
				urlAfter = value;
			}
		}

		private MessageBoard.Group _Group;
		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public MessageBoard.Group Group 
		{
			get
			{
				return _Group;
			}

			set
			{
				_Group = value;
			}
		}

		private MessageBoard.Language language;

		[Bindable(true), 
		Category("Appearance"), 
		DefaultValue("")] 
		public MessageBoard.Language Language 
		{
			get
			{
				return language;
			}

			set
			{
				language = value;
			}
		}

		/// <summary> 
		/// Render this control to the output parameter specified.
		/// </summary>
		/// <param name="output"> The HTML writer to write out to </param>
		//protected override void Render(HtmlTextWriter output)
		//{
		//	output.Write(Text);
		//}

		protected override void CreateChildControls() 
		{
			// The Group Items
            TextBox txtTitle = new TextBox();
			TextBox txtDescription = new TextBox();
			DropDownList cmbCategory = new DropDownList();
			Button cmdAdd = new Button();
			
			// Set the Title TextBox properties
			txtTitle.Text = _Group.Title;
			txtTitle.MaxLength = 100;
			txtTitle.Width = 500;
			txtTitle.ID="txtTitle";

			// Set the Description TextBox properties
			txtDescription.Text = _Group.Description;
			txtDescription.TextMode = TextBoxMode.MultiLine;
			txtDescription.Height = 100;
			txtDescription.Width = 500;
			txtDescription.ID = "txtDescription";

			// Set the Category DropDownList properties
			ListItem itmYes = new ListItem(Translate("Yes"), "true");
			ListItem itmNo = new ListItem(Translate("No"), "false");
			cmbCategory.Items.Add(itmYes);
			cmbCategory.Items.Add(itmNo);
			if(_Group.WithCategories==true)
                cmbCategory.SelectedIndex = 0;
			else
				cmbCategory.SelectedIndex = 1;
			cmbCategory.ID = "cmbCategory";
			
			// Set the Update button properties
			cmdAdd.Text = Translate("Update");

			// Put the contols in a table.
			HtmlTable table = new HtmlTable();
			table.Attributes.Add("width", "100%");
			HtmlTableRow tr0 = new HtmlTableRow();
			table.Controls.Add(tr0);
			HtmlTableCell td0 = new HtmlTableCell();
			tr0.Controls.Add(td0);
			td0.Attributes.Add("class", "Forum_Title1");
			td0.InnerHtml = Translate("Edit Group");

			HtmlTableRow tr01 = new HtmlTableRow();
			table.Controls.Add(tr01);
			HtmlTableCell td01 = new HtmlTableCell();
			tr01.Controls.Add(td01);
			td01.Attributes.Add("class", "Forum_Title4");

			// Table 2
			HtmlTable table2 = new HtmlTable();
			table2.Attributes.Add("width", "100%");

			// Title
			HtmlTableRow tr1 = new HtmlTableRow();
			table2.Controls.Add(tr1);
			HtmlTableCell td11 = new HtmlTableCell();
			HtmlTableCell td12 = new HtmlTableCell();
			td11.InnerHtml = Translate("Title")+":";
			td11.Attributes.Add("class", "Forum_Text");
			td12.Controls.Add(txtTitle);
			tr1.Controls.Add(td11);
			tr1.Controls.Add(td12);
			

			// Description
			HtmlTableRow tr2 = new HtmlTableRow();
			table2.Controls.Add(tr2);
			HtmlTableCell td21 = new HtmlTableCell();
			HtmlTableCell td22 = new HtmlTableCell();
			td21.InnerHtml = Translate("Description")+":";
			td21.Attributes.Add("class", "Forum_Text");
			td21.Attributes.Add("vAlign", "top");
			td22.Controls.Add(txtDescription);
			tr2.Controls.Add(td21);
			tr2.Controls.Add(td22);

			// WithCategories
			HtmlTableRow tr4 = new HtmlTableRow();
			table2.Controls.Add(tr4);
			HtmlTableCell td41 = new HtmlTableCell();
			HtmlTableCell td42 = new HtmlTableCell();
			td41.InnerHtml = Translate("Categories")+":";
			td41.Attributes.Add("class", "Forum_Text");
			td41.Attributes.Add("vAlign", "top");
			td42.Controls.Add(cmbCategory);
			tr4.Controls.Add(td41);
			tr4.Controls.Add(td42);

			// Button
			HtmlTableRow tr3 = new HtmlTableRow();
			table2.Controls.Add(tr3);
			HtmlTableCell td31 = new HtmlTableCell();
			HtmlTableCell td32 = new HtmlTableCell();
			td31.InnerHtml = "";
			td32.Controls.Add(cmdAdd);
			tr3.Controls.Add(td31);
			tr3.Controls.Add(td32);

			td01.Controls.Add(table2);

			HtmlTable tableOut = new HtmlTable();
			HtmlTableRow trOut = new HtmlTableRow();
			HtmlTableCell tdOut = new HtmlTableCell();
			tableOut.BgColor = "#000000";
			tableOut.Attributes.Add("width", "100%");
			tableOut.CellPadding = 0;
			tableOut.CellSpacing = 0;
			table.CellSpacing = 1;
			tdOut.Controls.Add(table);
			trOut.Controls.Add(tdOut);
			tableOut.Controls.Add(trOut);
			Controls.Add(tableOut);

			cmdAdd.Click += new EventHandler(cmdAdd_Click);
                  
		}

		public void cmdAdd_Click(Object sender, EventArgs e)
		{
			TextBox txtTitle = ((TextBox)FindControl("txtTitle"));
			TextBox txtDescription = ((TextBox)FindControl("txtDescription"));
			DropDownList cmbCategory = ((DropDownList)FindControl("cmbCategory"));
			if(txtTitle.Text!="" || ((TextBox)Controls[1]).Text!="")
			{
				_Group.Title= txtTitle.Text;
				_Group.Description = txtDescription.Text;
				_Group.WithCategories = Convert.ToBoolean(cmbCategory.SelectedValue);
				_Group.Update();

				txtTitle.Text="";
				txtDescription.Text="";
				this.EnsureChildControls();
				this.Page.Response.Redirect(urlAfter);
			}
		}

		private MessageBoard.CustomControl customControl = MessageBoard.CustomControl.Get("EditForum");

		private string Translate(string word)
		{
			return MessageBoard.Translation.Get(customControl, word, Language).Equal;
		}


	}
}
